function [u,x,y,yT,gamma,rho,eta] = DWempcGetSol(z,A,B,C,Upsilon,x0,N)
% extract EMPC information from DW solution values
M = length(A);
u = cell(M,1);
gamma = cell(M,1);
rho = cell(1,1);
eta = cell(M,1);
nx = cell(M,1);
nu = cell(M,1);
ny = cell(M,1);
for j = 1:M
    [nx{j},nu{j}] = size(B{j});
    ny{j} = size(C{j},1);
    u{j} = reshape(z{j}(1:N*nu{j}),nu{j},N);
    gamma{j} = reshape(z{j}(N*nu{j}+1:N*nu{j}+N*ny{j}),ny{j},N);
    eta{j} = reshape(z{j}(N*nu{j}+N*ny{j}+1:N*nu{j}+...
        N*ny{j}+N*nu{j}),nu{j},N);
end
rho{1} = z{M+1}(:);

yT = cell(1,1);
yT{1} = zeros(N+1,1);
x = cell(M,1);
y = cell(M,1);
for j = 1:M
    [x{j},y{j},yP] = SimulateSS(A{j},B{j},C{j},Upsilon{j},u{j},x0{j},N);
    yT{1} = yT{1} + yP';
end

function [x,y,yP,z,zP] = SimulateSS(A,B,C,Upsilon,u,x0,N)
x = zeros(size(A,1),N+1);
y = zeros(size(C,1),N+1);
z = zeros(size(C,1),N+1);
yP = zeros(1,N);
zP = zeros(1,N);
x(:,1) = x0;
for k = 1:N
    x(:,k+1) = A*x(:,k) + B*u(:,k);
    y(:,k) = C*x(:,k);
    yP(:,k) = Upsilon*y(:,k);
end
k = k + 1;
y(:,k) = C*x(:,k);
yP(:,k) = Upsilon*y(:,k);